@@ -2169,6 +2169,7 @@ mod mutable_vec {
             });
         }
 
+        // TODO can this be improved ?
         fn notify_with<B, C, D, E>(&mut self, value: B, mut clone: C, change: D, mut notify: E)
             where C: FnMut(&B) -> B,
                   D: FnOnce(&mut Self, B),
@@ -2310,6 +2311,16 @@ mod mutable_vec {
             self.remove_range(range.clone(), len);
             self.values.drain(range)
         }
+
+        fn truncate(&mut self, len: usize) {
+            let end = self.values.len();
+            let range = Range {
+                start: len,
+                end: end,
+            };
+            self.remove_range(range, end);
+            self.values.truncate(len)
+        }
     }
 
     impl<A: Copy> MutableVecState<A> {
@@ -2417,6 +2428,13 @@ mod mutable_vec {
                 |this, values| this.values = values,
                 |values| VecDiff::Replace { values });
         }
+
+        // TODO use reserve / extend
+        fn extend<I>(&mut self, iter: I) where I: IntoIterator<Item = A> {
+            for value in iter {
+                self.push_clone(value);
+            }
+        }
     }
 
 
@@ -2571,6 +2589,11 @@ mod mutable_vec {
             self.lock.drain(range)
         }
 
+        #[inline]
+        pub fn truncate(&mut self, len: usize) {
+            self.lock.truncate(len)
+        }
+
         pub fn reverse(&mut self) {
             let len = self.len();
 
@@ -2649,6 +2672,14 @@ mod mutable_vec {
 
     make_shared!(MutableVecLockMut<'a, A>, MutableVecLockMut<'b, B>);
 
+    // TODO extend_one and extend_reserve
+    impl<'a, A> Extend<A> for MutableVecLockMut<'a, A> where A: Clone {
+        #[inline]
+        fn extend<I>(&mut self, iter: I) where I: IntoIterator<Item = A> {
+            self.lock.extend(iter)
+        }
+    }
+
 
     // TODO get rid of the Arc
     // TODO impl some of the same traits as Vec
